@@ -7,6 +7,8 @@
 #include "AntiTrackingUtils.h"
 
 #include "AntiTrackingLog.h"
+#include "ContentBlockingAllowList.h"
+#include "HttpBaseChannel.h"
 #include "mozilla/BasePrincipal.h"
 #include "mozilla/Components.h"
 #include "mozilla/dom/BrowsingContext.h"
@@ -19,15 +21,18 @@
 #include "mozilla/net/NeckoChannelParams.h"
 #include "mozilla/PermissionManager.h"
 #include "mozIThirdPartyUtil.h"
-#include "nsEffectiveTLDService.h"
 #include "nsGlobalWindowInner.h"
+#include "nsHttpChannel.h"
 #include "nsIChannel.h"
 #include "nsICookieService.h"
+#include "nsIEffectiveTLDService.h"
 #include "nsIHttpChannel.h"
 #include "nsIPermission.h"
 #include "nsIURI.h"
 #include "nsNetUtil.h"
+#include "nsMixedContentBlocker.h"
 #include "nsPIDOMWindow.h"
+#include "nsQueryObject.h"
 #include "nsRFPService.h"
 #include "nsSandboxFlags.h"
 #include "nsScriptSecurityManager.h"
@@ -158,8 +163,8 @@ bool AntiTrackingUtils::CreateStorageFramePermissionKey(
 bool AntiTrackingUtils::CreateStorageRequestPermissionKey(
     nsIURI* aURI, nsACString& aPermissionKey) {
   MOZ_ASSERT(aPermissionKey.IsEmpty());
-  RefPtr<nsEffectiveTLDService> eTLDService =
-      nsEffectiveTLDService::GetInstance();
+  nsCOMPtr<nsIEffectiveTLDService> eTLDService =
+      mozilla::components::EffectiveTLD::Service();
   if (!eTLDService) {
     return false;
   }
@@ -210,7 +215,7 @@ bool AntiTrackingUtils::IsStorageAccessPermission(nsIPermission* aPermission,
 // static
 Maybe<size_t> AntiTrackingUtils::CountSitesAllowStorageAccess(
     nsIPrincipal* aPrincipal) {
-  PermissionManager* permManager = PermissionManager::GetInstance();
+  RefPtr<PermissionManager> permManager = PermissionManager::GetInstance();
   if (NS_WARN_IF(!permManager)) {
     return Nothing();
   }
@@ -284,10 +289,8 @@ Maybe<size_t> AntiTrackingUtils::CountSitesAllowStorageAccess(
 // static
 bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
                                                const nsAutoCString& aType,
-                                               bool aIsInPrivateBrowsing,
-                                               uint32_t* aRejectedReason,
-                                               uint32_t aBlockedReason) {
-  PermissionManager* permManager = PermissionManager::GetInstance();
+                                               bool aIsInPrivateBrowsing) {
+  RefPtr<PermissionManager> permManager = PermissionManager::GetInstance();
   if (NS_WARN_IF(!permManager)) {
     LOG(("Failed to obtain the permission manager"));
     return false;
@@ -354,9 +357,6 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
     }
 
     if (!found) {
-      if (aRejectedReason) {
-        *aRejectedReason = aBlockedReason;
-      }
       return false;
     }
   } else {
@@ -374,9 +374,6 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
         aPrincipal);
 
     if (result != nsIPermissionManager::ALLOW_ACTION) {
-      if (aRejectedReason) {
-        *aRejectedReason = aBlockedReason;
-      }
       return false;
     }
   }
@@ -386,9 +383,9 @@ bool AntiTrackingUtils::CheckStoragePermission(nsIPrincipal* aPrincipal,
 
 /* static */
 nsresult AntiTrackingUtils::TestStoragePermissionInParent(
-    nsIPrincipal* aTopPrincipal, nsIPrincipal* aPrincipal, bool* aResult) {
+    nsIPrincipal* aTopPrincipal, nsIPrincipal* aPrincipal, uint32_t* aResult) {
   NS_ENSURE_ARG(aResult);
-  *aResult = false;
+  *aResult = nsIPermissionManager::UNKNOWN_ACTION;
   NS_ENSURE_ARG(aTopPrincipal);
   NS_ENSURE_ARG(aPrincipal);
 
@@ -413,8 +410,8 @@ nsresult AntiTrackingUtils::TestStoragePermissionInParent(
       aTopPrincipal, requestPermissionKey, &access);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  if (access == nsIPermissionManager::ALLOW_ACTION) {
-    *aResult = true;
+  if (access != nsIPermissionManager::UNKNOWN_ACTION) {
+    *aResult = access;
     return NS_OK;
   }
 
@@ -423,7 +420,7 @@ nsresult AntiTrackingUtils::TestStoragePermissionInParent(
       aTopPrincipal, requestFramePermissionKey, &frameAccess);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  *aResult = frameAccess == nsIPermissionManager::ALLOW_ACTION;
+  *aResult = frameAccess;
   return NS_OK;
 }
 
@@ -512,6 +509,10 @@ AntiTrackingUtils::GetStoragePermissionStateInParent(nsIChannel* aChannel) {
     return nsILoadInfo::NoStoragePermission;
   }
 
+  if (targetPrincipal->IsSystemPrincipal()) {
+    return nsILoadInfo::HasStoragePermission;
+  }
+
   nsCOMPtr<nsIURI> trackingURI;
   rv = aChannel->GetURI(getter_AddRefs(trackingURI));
   if (NS_WARN_IF(NS_FAILED(rv))) {
@@ -522,7 +523,8 @@ AntiTrackingUtils::GetStoragePermissionStateInParent(nsIChannel* aChannel) {
       BasePrincipal::CreateContentPrincipal(trackingURI,
                                             loadInfo->GetOriginAttributes());
 
-  if (IsThirdPartyChannel(aChannel)) {
+  bool isThirdParty = IsThirdPartyChannel(aChannel);
+  if (isThirdParty) {
     nsAutoCString targetOrigin;
     nsAutoCString trackingOrigin;
     if (NS_FAILED(targetPrincipal->GetOriginNoSuffix(targetOrigin)) ||
@@ -530,57 +532,291 @@ AntiTrackingUtils::GetStoragePermissionStateInParent(nsIChannel* aChannel) {
       return nsILoadInfo::NoStoragePermission;
     }
 
+    // Check whether the third-party channel is on any allow lists. We check
+    // the partitioning exception list and the content blocking allow list.
     if (PartitioningExceptionList::Check(targetOrigin, trackingOrigin)) {
       return nsILoadInfo::StoragePermissionAllowListed;
     }
+
+    nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+    if (httpChannel && ContentBlockingAllowList::Check(httpChannel)) {
+      return nsILoadInfo::StoragePermissionAllowListed;
+    }
   }
 
   nsAutoCString type;
   AntiTrackingUtils::CreateStoragePermissionKey(trackingPrincipal, type);
 
-  uint32_t unusedReason = 0;
-
-  if (AntiTrackingUtils::CheckStoragePermission(targetPrincipal, type,
-                                                NS_UsePrivateBrowsing(aChannel),
-                                                &unusedReason, unusedReason)) {
+  if (AntiTrackingUtils::CheckStoragePermission(
+          targetPrincipal, type, NS_UsePrivateBrowsing(aChannel))) {
     return nsILoadInfo::HasStoragePermission;
   }
 
+  WindowContext* wc = bc->GetCurrentWindowContext();
+  if (!wc) {
+    return nsILoadInfo::NoStoragePermission;
+  }
+  WindowGlobalParent* wgp = wc->Canonical();
+  if (!wgp) {
+    return nsILoadInfo::NoStoragePermission;
+  }
+  nsIPrincipal* framePrincipal = wgp->DocumentPrincipal();
+  if (!framePrincipal) {
+    return nsILoadInfo::NoStoragePermission;
+  }
+
+  RefPtr<net::HttpBaseChannel> httpBaseChannel = do_QueryObject(aChannel);
+  if (httpBaseChannel && httpBaseChannel->HasRedirectTaintedOrigin()) {
+    return nsILoadInfo::NoStoragePermission;
+  }
+
   if (policyType == ExtContentPolicy::TYPE_SUBDOCUMENT) {
-    uint64_t targetWindowIdNoTop = bc->GetCurrentInnerWindowId();
+    // For loads of framed documents, we only use storage access
+    // if the load is the result of a same-origin, same-site-initiated
+    // navigation of the frame.
     uint64_t triggeringWindowId;
-    loadInfo->GetTriggeringWindowId(&triggeringWindowId);
-    bool triggeringStorageAccess;
-    loadInfo->GetTriggeringStorageAccess(&triggeringStorageAccess);
-    if (targetWindowIdNoTop == triggeringWindowId && triggeringStorageAccess &&
-        trackingPrincipal->Equals(loadInfo->TriggeringPrincipal())) {
-      return nsILoadInfo::HasStoragePermission;
-    }
-  } else if (!bc->IsTop()) {
-    // Only check the frame only permission if the channel is not in the top
-    // browsing context.
-    RefPtr<nsEffectiveTLDService> etld = nsEffectiveTLDService::GetInstance();
-    if (!etld) {
+    rv = loadInfo->GetTriggeringWindowId(&triggeringWindowId);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
       return nsILoadInfo::NoStoragePermission;
     }
-    nsCString trackingSite;
-    rv = etld->GetSite(trackingURI, trackingSite);
+    bool triggeringWindowHasStorageAccess;
+    rv =
+        loadInfo->GetTriggeringStorageAccess(&triggeringWindowHasStorageAccess);
     if (NS_WARN_IF(NS_FAILED(rv))) {
       return nsILoadInfo::NoStoragePermission;
     }
-    nsAutoCString type;
-    AntiTrackingUtils::CreateStorageFramePermissionKey(trackingSite, type);
 
-    if (AntiTrackingUtils::CheckStoragePermission(
-            targetPrincipal, type, NS_UsePrivateBrowsing(aChannel),
-            &unusedReason, unusedReason)) {
+    nsIScriptSecurityManager* ssm = nsContentUtils::GetSecurityManager();
+    RefPtr<nsIPrincipal> channelResultPrincipal;
+    rv = ssm->GetChannelResultPrincipal(aChannel,
+                                        getter_AddRefs(channelResultPrincipal));
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return nsILoadInfo::NoStoragePermission;
+    }
+    RefPtr<net::HttpBaseChannel> httpChannel = do_QueryObject(aChannel);
+    bool crossSiteInitiated = false;
+    if (bc && bc->GetParent()->GetCurrentWindowContext()) {
+      RefPtr<WindowGlobalParent> triggeringWGP =
+          WindowGlobalParent::GetByInnerWindowId(triggeringWindowId);
+      if (triggeringWGP && triggeringWGP->DocumentPrincipal()) {
+        rv = triggeringWGP->DocumentPrincipal()->IsThirdPartyPrincipal(
+            channelResultPrincipal, &crossSiteInitiated);
+        if (NS_FAILED(rv)) {
+          crossSiteInitiated = false;
+        }
+      }
+    }
+
+    if (!crossSiteInitiated && triggeringWindowHasStorageAccess &&
+        trackingPrincipal->Equals(framePrincipal) && httpChannel &&
+        !httpChannel->HasRedirectTaintedOrigin()) {
       return nsILoadInfo::HasStoragePermission;
     }
+  } else if (!bc->IsTop()) {
+    // For subframe resources, check if the document has storage access
+    // and that the resource being loaded is same-site to the page.
+    bool isThirdParty = true;
+    nsresult rv = framePrincipal->IsThirdPartyURI(trackingURI, &isThirdParty);
+    if (NS_SUCCEEDED(rv) && wc->GetUsingStorageAccess() && !isThirdParty) {
+      return nsILoadInfo::HasStoragePermission;
+    }
+  }
+
+  // The remaining part of the function is for determining whether storage
+  // access could be granted using Storage-Access-Headers. And granting it
+  // if instructed by the server via the "Activate-Storage-Access"-header.
+  // Storage-Access headers are only sent in secure context
+  if (!nsMixedContentBlocker::IsPotentiallyTrustworthyOrigin(trackingURI)) {
+    return nsILoadInfo::NoStoragePermission;
+  }
+
+  // In case Storage-Access was granted to the origin prior with the
+  // Storage-Access-API and the permission still exists, the website can
+  // activate Storage-Access with Storage-Access-Headers.
+  uint32_t result = 0;
+  rv = AntiTrackingUtils::TestStoragePermissionInParent(
+      targetPrincipal, trackingPrincipal, &result);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return nsILoadInfo::NoStoragePermission;
+  }
+  if (result == nsIPermissionManager::ALLOW_ACTION) {
+    if (RefPtr<net::nsHttpChannel> httpChannel = do_QueryObject(aChannel)) {
+      if (httpChannel->StorageAccessReloadedChannel()) {
+        return nsILoadInfo::HasStoragePermission;
+      } else {
+        return nsILoadInfo::InactiveStoragePermission;
+      }
+    }
+  }
+
+  // In the ABA-case, A can also get storage-access automatically via
+  // Storage-Access-Headers.
+  if (isThirdParty) {
+    if (RefPtr<net::nsHttpChannel> httpChannel = do_QueryObject(aChannel)) {
+      // Determine whether we are in ABA or AB case, erroring on AB side
+      bool isAB = true;
+      rv = targetPrincipal->IsThirdPartyURI(trackingURI, &isAB);
+      if (NS_FAILED(rv)) {
+        return nsILoadInfo::NoStoragePermission;
+      }
+      if (isAB) {
+        // Third party resource, that could potentially have storage access
+        // Sending "Sec-Fetch-Storage-Access: none"
+        return nsILoadInfo::DisabledStoragePermission;
+      } else {
+        if (httpChannel->StorageAccessReloadedChannel()) {
+          // The server sent a retry response after we sent the "inactive"
+          // activate storage-access
+          // Sending "Sec-Fetch-Storage-Access: active"
+          return nsILoadInfo::HasStoragePermission;
+        } else {
+          // The site could has implicit storage-access due to ABA scenario.
+          // Let the server know that replying with "retry" will lead to us
+          // reloading the channel with storage-access enabled.
+          // Sending "Sec-Fetch-Storage-Access: inactive"
+          return nsILoadInfo::InactiveStoragePermission;
+        }
+      }
+    }
   }
 
   return nsILoadInfo::NoStoragePermission;
 }
 
+// static
+nsresult AntiTrackingUtils::ActivateStoragePermissionStateInParent(
+    nsIChannel* aChannel) {
+  NS_ENSURE_ARG_POINTER(aChannel);
+  MOZ_DIAGNOSTIC_ASSERT(XRE_IsParentProcess());
+
+  if (GetStoragePermissionStateInParent(aChannel) !=
+      nsILoadInfo::InactiveStoragePermission) {
+    return NS_ERROR_FAILURE;
+  }
+
+  nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
+  if (NS_WARN_IF(!loadInfo)) {
+    return NS_ERROR_FAILURE;
+  }
+
+  RefPtr<BrowsingContext> bc;
+  nsresult rv = loadInfo->GetTargetBrowsingContext(getter_AddRefs(bc));
+  if (NS_WARN_IF(NS_FAILED(rv)) || !bc) {
+    return NS_ERROR_FAILURE;
+  }
+
+  WindowContext* wc = bc->GetCurrentWindowContext();
+  if (NS_WARN_IF(!wc)) {
+    return NS_ERROR_FAILURE;
+  }
+
+#ifdef DEBUG
+  // We are only allowed to transition from "Inactive" to "Has". Parent function
+  // should check this condition, but check here again to make extra sure.
+  nsILoadInfo::StoragePermissionState currentStorageAccess =
+      loadInfo->GetStoragePermission();
+  MOZ_ASSERT(currentStorageAccess == nsILoadInfo::InactiveStoragePermission);
+#endif
+
+  // Allow accessing unpartitioned cookies
+  MOZ_TRY(loadInfo->SetStoragePermission(nsILoadInfo::HasStoragePermission));
+  MOZ_TRY(wc->SetUsingStorageAccess(true));
+
+  return NS_OK;
+}
+
+bool AntiTrackingUtils::ProcessStorageAccessHeadersShouldRetry(
+    nsIChannel* aChannel) {
+  bool ShouldRetry = false;
+  nsresult rv = ProcessStorageAccessHeaders(aChannel, &ShouldRetry);
+  return NS_SUCCEEDED(rv) && ShouldRetry;
+}
+
+nsresult AntiTrackingUtils::ProcessStorageAccessHeaders(nsIChannel* aChannel,
+                                                        bool* aOutRetry) {
+  NS_ENSURE_ARG_POINTER(aChannel);
+  NS_ENSURE_ARG_POINTER(aOutRetry);
+  *aOutRetry = false;
+  if (!StaticPrefs::dom_storage_access_enabled() ||
+      !StaticPrefs::dom_storage_access_headers_enabled()) {
+    return NS_OK;
+  }
+
+  nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
+  NS_ENSURE_TRUE(httpChannel, NS_ERROR_FAILURE);
+
+  // only continue processing if we got an Activate-Storage-Access response
+  // header
+  nsAutoCString activate;
+  MOZ_TRY(
+      httpChannel->GetResponseHeader("Activate-Storage-Access"_ns, activate));
+
+  nsCOMPtr<nsILoadInfo> loadInfo;
+  MOZ_TRY(aChannel->GetLoadInfo(getter_AddRefs(loadInfo)));
+
+  // 1. If request's credentials mode is not "include", return failure.
+  uint32_t cookiePolicy = 0;
+  MOZ_TRY(loadInfo->GetCookiePolicy(&cookiePolicy));
+  if (cookiePolicy != nsILoadInfo::SEC_COOKIES_INCLUDE) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // 2. If request's `eligible for storage-access` is "eligible", return
+  // failure. When storage-access is already granted, no need to process further
+  nsILoadInfo::StoragePermissionState storageAccess =
+      AntiTrackingUtils::GetStoragePermissionStateInParent(aChannel);
+  if (storageAccess != nsILoadInfo::InactiveStoragePermission) {
+    return NS_ERROR_FAILURE;
+  }
+
+  // 3. Let storageAccessStatus be request's storage access status.
+  nsAutoCString storageAccessStatus;
+
+  // 4. If storageAccessStatus is not "inactive", return failure.
+  MOZ_TRY(httpChannel->GetRequestHeader("Sec-Fetch-Storage-Access"_ns,
+                                        storageAccessStatus));
+  if (!storageAccessStatus.EqualsLiteral("inactive")) {
+    return NS_ERROR_FAILURE;
+  }
+  net::ActivateStorageAccess asa =
+      MOZ_TRY(net::ParseActivateStorageAccess(activate));
+
+  switch (asa.variant) {
+    case net::ActivateStorageAccessVariant::Load: {
+      // TODO: only do on document channels / load?
+      auto policyType = loadInfo->GetExternalContentPolicyType();
+      if (policyType != ExtContentPolicy::TYPE_SUBDOCUMENT) {
+        return NS_ERROR_FAILURE;
+      }
+      MOZ_TRY(
+          AntiTrackingUtils::ActivateStoragePermissionStateInParent(aChannel));
+      return NS_OK;
+    }
+    case net::ActivateStorageAccessVariant::RetryOrigin: {
+      // check whether specified allowedOrigin allows this retry
+      // parse origin
+      nsCOMPtr<nsIURI> allowedOrigin;
+      MOZ_TRY(NS_NewURI(getter_AddRefs(allowedOrigin), asa.origin.get()));
+
+      nsIPrincipal* loadingPrincipal = loadInfo->GetLoadingPrincipal();
+      if (!loadingPrincipal) {
+        return NS_ERROR_FAILURE;
+      }
+      if (!loadingPrincipal->IsSameOrigin(allowedOrigin)) {
+        return NS_ERROR_FAILURE;
+      }
+      *aOutRetry = true;
+      return NS_OK;
+    }
+    case net::ActivateStorageAccessVariant::RetryAny:
+      // no checks on origin necessary
+      *aOutRetry = true;
+      return NS_OK;
+  }
+  MOZ_ASSERT(false, "Invalid enum variant");
+  return NS_ERROR_FAILURE;
+}
+
 uint64_t AntiTrackingUtils::GetTopLevelAntiTrackingWindowId(
     BrowsingContext* aBrowsingContext) {
   MOZ_ASSERT(aBrowsingContext);
@@ -734,6 +970,43 @@ AntiTrackingUtils::GetTopWindowExcludingExtensionAccessibleContentFrames(
   return prev.forget();
 }
 
+/* static */
+nsresult AntiTrackingUtils::IsThirdPartyToPartitionKeySite(
+    nsIChannel* aChannel, const nsCOMPtr<nsIURI>& aURI, bool* aIsThirdParty) {
+  MOZ_ASSERT(XRE_IsParentProcess());
+  NS_ENSURE_ARG(aChannel);
+  NS_ENSURE_ARG(aURI);
+  NS_ENSURE_ARG(aIsThirdParty);
+  nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
+  nsCOMPtr<nsICookieJarSettings> cjs;
+  nsresult rv = loadInfo->GetCookieJarSettings(getter_AddRefs(cjs));
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsAutoString partitionKey;
+  rv = cjs->GetPartitionKey(partitionKey);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsAutoString scheme, host;
+  int32_t _unused1;
+  bool _unused2;
+  if (!OriginAttributes::ParsePartitionKey(partitionKey, scheme, host, _unused1,
+                                           _unused2)) {
+    return NS_ERROR_FAILURE;
+  }
+  if (host.IsEmpty()) {
+    return NS_ERROR_FAILURE;
+  }
+  nsAutoString partitionKeySite = scheme + u"://"_ns + host;
+  nsCOMPtr<nsIURI> partitionKeySiteURI;
+  rv = NS_NewURI(getter_AddRefs(partitionKeySiteURI), partitionKeySite);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  nsCOMPtr<mozIThirdPartyUtil> thirdPartyUtil =
+      components::ThirdPartyUtil::Service();
+  return thirdPartyUtil->IsThirdPartyURI(aURI, partitionKeySiteURI,
+                                         aIsThirdParty);
+}
+
 /* static */
 void AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(nsIChannel* aChannel) {
   MOZ_ASSERT(aChannel);
@@ -753,6 +1026,9 @@ void AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(nsIChannel* aChannel) {
 
   RefPtr<BrowsingContext> bc;
   loadInfo->GetBrowsingContext(getter_AddRefs(bc));
+  if (!bc) {
+    bc = loadInfo->GetWorkerAssociatedBrowsingContext();
+  }
 
   nsCOMPtr<nsIURI> uri;
   Unused << aChannel->GetURI(getter_AddRefs(uri));
@@ -772,13 +1048,29 @@ void AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(nsIChannel* aChannel) {
       return;
     }
 
-    // We turn to check the loading principal if there is no browsing context.
-    auto* loadingPrincipal =
-        BasePrincipal::Cast(loadInfo->GetLoadingPrincipal());
+    // We turn to check the top-level principal if there is no browsing context.
+    auto* principal = BasePrincipal::Cast(loadInfo->GetTopLevelPrincipal());
 
-    if (uri && loadingPrincipal) {
+    // If we don't have the top level principal, we try to compare directly to
+    // the scheme in the partition key. Failing that, we fall back to
+    // comparing to the loading principal. But we can only do this if this
+    // channel has a URI. If we have neither a channel URI or the browsing
+    // context, we can do nothing.
+    if (uri) {
+      if (!principal) {
+        bool isThirdParty = true;
+        nsresult rv =
+            IsThirdPartyToPartitionKeySite(aChannel, uri, &isThirdParty);
+        if (NS_SUCCEEDED(rv)) {
+          loadInfo->SetIsThirdPartyContextToTopWindow(isThirdParty);
+          return;
+        }
+      }
+      principal = BasePrincipal::Cast(loadInfo->GetLoadingPrincipal());
+    }
+    if (principal) {
       bool isThirdParty = true;
-      nsresult rv = loadingPrincipal->IsThirdPartyURI(uri, &isThirdParty);
+      nsresult rv = principal->IsThirdPartyURI(uri, &isThirdParty);
 
       if (NS_SUCCEEDED(rv)) {
         loadInfo->SetIsThirdPartyContextToTopWindow(isThirdParty);
@@ -804,7 +1096,8 @@ void AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(nsIChannel* aChannel) {
     // whether the page is third-party, so we use channel result principal
     // instead. By doing this, an the resource inherits the principal from
     // its parent is considered not a third-party.
-    if (NS_IsAboutBlank(uri) || NS_IsAboutSrcdoc(uri)) {
+    if (NS_IsAboutBlank(uri) || NS_IsAboutSrcdoc(uri) ||
+        uri->SchemeIs("blob")) {
       nsIScriptSecurityManager* ssm = nsContentUtils::GetSecurityManager();
       if (NS_WARN_IF(!ssm)) {
         return;
@@ -830,10 +1123,17 @@ void AntiTrackingUtils::ComputeIsThirdPartyToTopWindow(nsIChannel* aChannel) {
 bool AntiTrackingUtils::IsThirdPartyChannel(nsIChannel* aChannel) {
   MOZ_ASSERT(aChannel);
 
-  // We only care whether the channel is 3rd-party with respect to
-  // the top-level.
-  nsCOMPtr<nsILoadInfo> loadInfo = aChannel->LoadInfo();
-  return loadInfo->GetIsThirdPartyContextToTopWindow();
+  nsCOMPtr<mozIThirdPartyUtil> tpuService =
+      mozilla::components::ThirdPartyUtil::Service();
+  if (!tpuService) {
+    return true;
+  }
+  bool thirdParty = true;
+  nsresult rv = tpuService->IsThirdPartyChannel(aChannel, nullptr, &thirdParty);
+  if (NS_FAILED(rv)) {
+    return true;
+  }
+  return thirdParty;
 }
 
 /* static */
@@ -886,19 +1186,54 @@ bool AntiTrackingUtils::IsThirdPartyWindow(nsPIDOMWindowInner* aWindow,
 /* static */
 bool AntiTrackingUtils::IsThirdPartyDocument(Document* aDocument) {
   MOZ_ASSERT(aDocument);
-  if (!aDocument->GetChannel()) {
-    // If we can't get the channel from the document, i.e. initial about:blank
-    // page, we use the browsingContext of the document to check if it's in the
-    // third-party context. If the browsing context is still not available, we
-    // will treat the window as third-party.
-    RefPtr<BrowsingContext> bc = aDocument->GetBrowsingContext();
-    return bc ? IsThirdPartyContext(bc) : true;
+
+  if (aDocument->IsTopLevelContentDocument()) {
+    return false;
   }
 
-  // We only care whether the channel is 3rd-party with respect to
-  // the top-level.
-  nsCOMPtr<nsILoadInfo> loadInfo = aDocument->GetChannel()->LoadInfo();
-  return loadInfo->GetIsThirdPartyContextToTopWindow();
+  nsCOMPtr<mozIThirdPartyUtil> tpuService =
+      mozilla::components::ThirdPartyUtil::Service();
+  if (!tpuService) {
+    return true;
+  }
+  bool thirdParty = true;
+
+  if (aDocument->GetSandboxFlags() & SANDBOXED_ORIGIN) {
+    return true;
+  }
+
+  if (!aDocument->GetChannel()) {
+    // If we can't get the channel from the document, i.e. initial about:blank
+    // page, we first check if we should inherit from the parent document. If
+    // the principal of the document is the same as the principal of the parent
+    // document, we inherit the third-party status from the parent document.
+    // Otherwise, we use the browsingContext of the document to check if it's
+    // in the third-party context. If the browsing context is still not
+    // available, we will treat the window as third-party.
+    //
+    // Note that we cannot directly use the browsingContext here. The
+    // browsingContext of a mixed but same baseDomain context with top level
+    // will be considered as third-party because they are not in the same
+    // content process. However, it should be considered as first-party.
+    RefPtr<Document> parentDoc = aDocument->GetInProcessParentDocument();
+    if (parentDoc &&
+        aDocument->NodePrincipal()->Equals(parentDoc->NodePrincipal())) {
+      return IsThirdPartyDocument(parentDoc);
+    }
+
+    RefPtr<BrowsingContext> bc = aDocument->GetBrowsingContext();
+    if (bc && bc->IsInProcess()) {
+      return IsThirdPartyContext(bc);
+    }
+    return true;
+  }
+
+  nsresult rv = tpuService->IsThirdPartyChannel(aDocument->GetChannel(),
+                                                nullptr, &thirdParty);
+  if (NS_FAILED(rv)) {
+    return true;
+  }
+  return thirdParty;
 }
 
 /* static */
@@ -906,41 +1241,47 @@ bool AntiTrackingUtils::IsThirdPartyContext(BrowsingContext* aBrowsingContext) {
   MOZ_ASSERT(aBrowsingContext);
   MOZ_ASSERT(aBrowsingContext->IsInProcess());
 
-  if (aBrowsingContext->IsTopContent()) {
-    return false;
-  }
-
-  // If the top browsing context is not in the same process, it's cross-origin.
-  if (!aBrowsingContext->Top()->IsInProcess()) {
-    return true;
-  }
-
+  // iframes with SANDBOX_ORIGIN are always third-party contexts
+  // because they are a unique origin
   nsIDocShell* docShell = aBrowsingContext->GetDocShell();
   if (!docShell) {
     return true;
   }
   Document* doc = docShell->GetExtantDocument();
-  if (!doc) {
+  if (!doc || doc->GetSandboxFlags() & SANDBOXED_ORIGIN) {
     return true;
   }
   nsIPrincipal* principal = doc->NodePrincipal();
 
-  nsIDocShell* topDocShell = aBrowsingContext->Top()->GetDocShell();
-  if (!topDocShell) {
-    return true;
+  BrowsingContext* traversingParent = aBrowsingContext->GetParent();
+  while (traversingParent) {
+    // If the parent browsing context is not in the same process, it's
+    // cross-origin.
+    if (!traversingParent->IsInProcess()) {
+      return true;
+    }
+
+    nsIDocShell* parentDocShell = traversingParent->GetDocShell();
+    if (!parentDocShell) {
+      return true;
+    }
+    Document* parentDoc = parentDocShell->GetDocument();
+    if (!parentDoc || parentDoc->GetSandboxFlags() & SANDBOXED_ORIGIN) {
+      return true;
+    }
+    nsIPrincipal* parentPrincipal = parentDoc->NodePrincipal();
+
+    auto* parentBasePrin = BasePrincipal::Cast(parentPrincipal);
+    bool isThirdParty = true;
+
+    parentBasePrin->IsThirdPartyPrincipal(principal, &isThirdParty);
+    if (isThirdParty) {
+      return true;
+    }
+
+    traversingParent = traversingParent->GetParent();
   }
-  Document* topDoc = topDocShell->GetDocument();
-  if (!topDoc) {
-    return true;
-  }
-  nsIPrincipal* topPrincipal = topDoc->NodePrincipal();
-
-  auto* topBasePrin = BasePrincipal::Cast(topPrincipal);
-  bool isThirdParty = true;
-
-  topBasePrin->IsThirdPartyPrincipal(principal, &isThirdParty);
-
-  return isThirdParty;
+  return false;
 }
 
 /* static */
@@ -973,6 +1314,29 @@ void AntiTrackingUtils::UpdateAntiTrackingInfoForChannel(nsIChannel* aChannel) {
   Unused << loadInfo->SetStoragePermission(
       AntiTrackingUtils::GetStoragePermissionStateInParent(aChannel));
 
+  // Note that we need to put this after computing the IsThirdPartyToTopWindow
+  // flag because it will be used when getting the granular fingerprinting
+  // protections.
+  Maybe<RFPTargetSet> overriddenFingerprintingSettings =
+      nsRFPService::GetOverriddenFingerprintingSettingsForChannel(aChannel);
+
+  if (overriddenFingerprintingSettings) {
+    loadInfo->SetOverriddenFingerprintingSettings(
+        overriddenFingerprintingSettings.ref());
+  }
+#ifdef DEBUG
+  static_cast<mozilla::net::LoadInfo*>(loadInfo.get())
+      ->MarkOverriddenFingerprintingSettingsAsSet();
+#endif
+
+  nsCOMPtr<nsICookieJarSettings> cookieJarSettings;
+  Unused << loadInfo->GetCookieJarSettings(getter_AddRefs(cookieJarSettings));
+  // Subresources (including subdocuments) may have a different partition key,
+  // particularly one without or with the same-site bit. We have to update that
+  // here.
+  net::CookieJarSettings::Cast(cookieJarSettings)
+      ->UpdatePartitionKeyForDocumentLoadedByChannel(aChannel);
+
   // We only update the IsOnContentBlockingAllowList flag and the partition key
   // for the top-level http channel.
   //
@@ -982,15 +1346,12 @@ void AntiTrackingUtils::UpdateAntiTrackingInfoForChannel(nsIChannel* aChannel) {
   //
   // The partition key is computed based on the site, so it's no point to set it
   // for channels other than http channels.
+  ExtContentPolicyType contentType = loadInfo->GetExternalContentPolicyType();
   nsCOMPtr<nsIHttpChannel> httpChannel = do_QueryInterface(aChannel);
-  if (!httpChannel || loadInfo->GetExternalContentPolicyType() !=
-                          ExtContentPolicy::TYPE_DOCUMENT) {
+  if (!httpChannel || contentType != ExtContentPolicy::TYPE_DOCUMENT) {
     return;
   }
 
-  nsCOMPtr<nsICookieJarSettings> cookieJarSettings;
-  Unused << loadInfo->GetCookieJarSettings(getter_AddRefs(cookieJarSettings));
-
   // Update the IsOnContentBlockingAllowList flag in the CookieJarSettings
   // if this is a top level loading. For sub-document loading, this flag
   // would inherit from the parent.
@@ -1001,7 +1362,7 @@ void AntiTrackingUtils::UpdateAntiTrackingInfoForChannel(nsIChannel* aChannel) {
   // propagated to non-top level loads via CookieJarSetting.
   nsCOMPtr<nsIURI> uri;
   Unused << aChannel->GetURI(getter_AddRefs(uri));
-  net::CookieJarSettings::Cast(cookieJarSettings)->SetPartitionKey(uri);
+  net::CookieJarSettings::Cast(cookieJarSettings)->SetPartitionKey(uri, false);
 
   // Generate the fingerprinting randomization key for top-level loads. The key
   // will automatically be propagated to sub loads.
