@@ -27,6 +27,29 @@ const TEXT_TRACK_FONT_SIZE_PREF =
   "media.videocontrols.picture-in-picture.display-text-tracks.size";
 const IMPROVED_CONTROLS_ENABLED_PREF =
   "media.videocontrols.picture-in-picture.improved-video-controls.enabled";
+const SEETHROUGH_MODE_ENABLED_PREF =
+  "media.videocontrols.picture-in-picture.seethrough-mode.enabled";
+
+/**
+ * The "showing" attribute means that we intentionally want to show controls
+ * on the PiP window. Examples include:
+ * - temporarily revealing PiP controls after initializing a PiP window.
+ * - revealing PiP controls after pausing the video.
+ *
+ * We remove the attribute once we're ready to hide controls on the window.
+ */
+const SHOWING_ATTRIBUTE = "showing";
+/**
+ * The "keying" attribute means that the controls have keyboard focus. Set this
+ * pref to keep controls visible on the PiP window until focus is gone.
+ */
+const KEYING_ATTRIBUTE = "keying";
+/**
+ * The "donthide" attribute is used to keep controls visible while users
+ * interact with the PiP CC settings menu. This ensures that we don't hide
+ * controls while the menu is still in use.
+ */
+const DONTHIDE_ATTRIBUTE = "donthide";
 
 // Time to fade the Picture-in-Picture video controls after first opening.
 const CONTROLS_FADE_TIMEOUT_MS = 3000;
@@ -43,7 +66,7 @@ const BOTTOM_LEFT_QUADRANT = 3;
 const BOTTOM_RIGHT_QUADRANT = 4;
 
 /**
- * Public function to be called from PictureInPicture.jsm. This is the main
+ * Public function to be called from PictureInPicture.sys.mjs. This is the main
  * entrypoint for initializing the player window.
  *
  * @param {Number} id
@@ -54,12 +77,12 @@ const BOTTOM_RIGHT_QUADRANT = 4;
  *    A reference to the video element that a Picture-in-Picture window
  *    is being created for
  */
-function setupPlayer(id, wgp, videoRef) {
-  Player.init(id, wgp, videoRef);
+function setupPlayer(id, wgp, videoRef, autoFocus) {
+  Player.init(id, wgp, videoRef, autoFocus);
 }
 
 /**
- * Public function to be called from PictureInPicture.jsm. This update the
+ * Public function to be called from PictureInPicture.sys.mjs. This update the
  * controls based on whether or not the video is playing.
  *
  * @param {Boolean} isPlaying
@@ -70,7 +93,7 @@ function setIsPlayingState(isPlaying) {
 }
 
 /**
- * Public function to be called from PictureInPicture.jsm. This update the
+ * Public function to be called from PictureInPicture.sys.mjs. This update the
  * controls based on whether or not the video is muted.
  *
  * @param {Boolean} isMuted
@@ -119,14 +142,20 @@ function setVolume(volume) {
   Player.setVolume(volume);
 }
 
+function closeFromForeground() {
+  Player.closeFromForeground();
+}
+
 /**
  * The Player object handles initializing the player, holds state, and handles
  * events for updating state.
  */
 let Player = {
+  _isInitialized: false,
   WINDOW_EVENTS: [
     "click",
     "contextmenu",
+    "command",
     "dblclick",
     "keydown",
     "mouseup",
@@ -174,6 +203,11 @@ let Player = {
    */
   deferredResize: null,
 
+  /**
+   * Set a shortcut that can be used for unpiping without pausing
+   */
+  isUnpipWithoutPauseShortcut: e => e.shiftKey === true,
+
   /**
    * Initializes the player browser, and sets up the initial state.
    *
@@ -182,10 +216,12 @@ let Player = {
    * @param {WindowGlobalParent} wgp
    *   The WindowGlobalParent that is hosting the originating video.
    * @param {ContentDOMReference} videoRef
-   *    A reference to the video element that a Picture-in-Picture window
-   *    is being created for
+   *   A reference to the video element that a Picture-in-Picture window
+   *   is being created for
+   * @param {boolean} autoFocus
+   *   Autofocus the PiP window
    */
-  init(id, wgp, videoRef) {
+  init(id, wgp, videoRef, autoFocus) {
     this.id = id;
 
     // State for whether or not we are adjusting the time via the scrubber
@@ -244,10 +280,10 @@ let Player = {
       this.audioScrubbing = true;
       this.handleAudioScrubbing(event.target.value);
     });
-    this.audioScrubber.addEventListener("change", event => {
+    this.audioScrubber.addEventListener("change", () => {
       this.audioScrubbing = false;
     });
-    this.audioScrubber.addEventListener("pointerdown", event => {
+    this.audioScrubber.addEventListener("pointerdown", () => {
       if (this.isMuted) {
         this.audioScrubber.max = 1;
       }
@@ -309,8 +345,8 @@ let Player = {
     this.resizeDebouncer = new DeferredTask(() => {
       this.alignEndControlsButtonTooltips();
       this.recordEvent("resize", {
-        width: window.outerWidth.toString(),
-        height: window.outerHeight.toString(),
+        width: window.outerWidth,
+        height: window.outerHeight,
       });
     }, RESIZE_DEBOUNCE_RATE_MS);
 
@@ -319,9 +355,11 @@ let Player = {
     // alwaysontop windows are not focused by default, so we have to do it
     // ourselves. We use requestAnimationFrame since we have to wait until the
     // window is visible before it can focus.
-    window.requestAnimationFrame(() => {
-      window.focus();
-    });
+    if (autoFocus) {
+      window.requestAnimationFrame(() => {
+        window.focus();
+      });
+    }
 
     let fontSize = Services.prefs.getCharPref(
       TEXT_TRACK_FONT_SIZE_PREF,
@@ -334,6 +372,13 @@ let Player = {
     } else {
       document.querySelector("#medium").checked = "true";
     }
+
+    // In see-through mode the PiP window is made semi-transparent on hover.
+    if (Services.prefs.getBoolPref(SEETHROUGH_MODE_ENABLED_PREF, false)) {
+      document.documentElement.classList.add("seethrough-mode");
+    }
+
+    this._isInitialized = true;
   },
 
   uninit() {
@@ -356,11 +401,22 @@ let Player = {
         // Don't run onClick if middle or right click is pressed respectively
         if (event.button !== 1 && event.button !== 2) {
           this.onClick(event);
-          this.controls.removeAttribute("keying");
+          this.controls.removeAttribute(KEYING_ATTRIBUTE);
         }
         break;
       }
 
+      case "command":
+        switch (event.target.id) {
+          case "View:PictureInPicture":
+            this.onCommand(event);
+            break;
+          case "View:Fullscreen":
+            this.fullscreenModeToggle(event);
+            break;
+        }
+        break;
+
       case "contextmenu": {
         event.preventDefault();
         break;
@@ -373,7 +429,7 @@ let Player = {
 
       case "keydown": {
         if (event.keyCode == KeyEvent.DOM_VK_TAB) {
-          this.controls.setAttribute("keying", true);
+          this.controls.setAttribute(KEYING_ATTRIBUTE, true);
           this.showVideoControls();
         } else if (event.keyCode == KeyEvent.DOM_VK_ESCAPE) {
           let isSettingsPanelInFocus = this.settingsPanel.contains(
@@ -393,7 +449,7 @@ let Player = {
             document.exitFullscreen();
           } else {
             // We handle the ESC key, as an intent to leave the picture-in-picture modus
-            this.onClose();
+            this.onClose(this.isUnpipWithoutPauseShortcut(event));
           }
         } else if (
           Services.prefs.getBoolPref(KEYBOARD_CONTROLS_ENABLED_PREF, false) &&
@@ -463,8 +519,8 @@ let Player = {
           this.actor.sendAsyncMessage("PictureInPicture:ExitFullscreen", {
             isFullscreen: this.isFullscreen,
             isVideoControlsShowing:
-              !!this.controls.getAttribute("showing") ||
-              !!this.controls.getAttribute("keying"),
+              this.controls.hasAttribute(SHOWING_ATTRIBUTE) ||
+              this.controls.hasAttribute(KEYING_ATTRIBUTE),
             playerBottomControlsDOMRect:
               this.controlsBottom.getBoundingClientRect(),
           });
@@ -478,7 +534,7 @@ let Player = {
       }
 
       case "oop-browser-crashed": {
-        this.closePipWindow({ reason: "browser-crash" });
+        this.closePipWindow({ reason: "BrowserCrash" });
         break;
       }
 
@@ -641,7 +697,7 @@ let Player = {
       }
 
       case "close": {
-        this.onClose();
+        this.onClose(this.isUnpipWithoutPauseShortcut(event));
         break;
       }
 
@@ -685,7 +741,7 @@ let Player = {
       case "fullscreen": {
         this.fullscreenModeToggle();
         this.recordEvent("fullscreen", {
-          enter: (!this.isFullscreen).toString(),
+          enter: !this.isFullscreen,
         });
         break;
       }
@@ -724,25 +780,21 @@ let Player = {
     if (options?.forceHide || settingsPanelVisible) {
       this.settingsPanel.classList.add("hide");
       this.closedCaptionButton.setAttribute("aria-expanded", false);
-      this.controls.removeAttribute("donthide");
+      this.controls.removeAttribute(DONTHIDE_ATTRIBUTE);
 
       if (
-        this.controls.getAttribute("keying") ||
+        this.controls.hasAttribute(KEYING_ATTRIBUTE) ||
         this.isCurrentHover ||
-        this.controls.getAttribute("showing")
+        this.controls.hasAttribute(SHOWING_ATTRIBUTE)
       ) {
         return;
       }
 
-      this.actor.sendAsyncMessage("PictureInPicture:HideVideoControls", {
-        isFullscreen: this.isFullscreen,
-        isVideoControlsShowing: false,
-        playerBottomControlsDOMRect: null,
-      });
+      this.hideVideoControls();
     } else {
       this.settingsPanel.classList.remove("hide");
       this.closedCaptionButton.setAttribute("aria-expanded", true);
-      this.controls.setAttribute("donthide", true);
+      this.controls.setAttribute(DONTHIDE_ATTRIBUTE, true);
       this.showVideoControls();
 
       if (options?.isKeyboard) {
@@ -751,11 +803,23 @@ let Player = {
     }
   },
 
-  onClose() {
-    this.actor.sendAsyncMessage("PictureInPicture:Pause", {
-      reason: "pip-closed",
+  onClose(bypassPause = false) {
+    // By default, we want to pause the video on close, unless the user
+    // used the assigned isUnpipWithoutPauseShortcut
+    if (!bypassPause) {
+      this.actor.sendAsyncMessage("PictureInPicture:Pause", {
+        reason: "pip-closed",
+      });
+    }
+
+    this.closePipWindow({ reason: "CloseButton" });
+  },
+
+  closeFromForeground() {
+    PictureInPicture.closeSinglePipWindow({
+      reason: "Foregrounded",
+      actorRef: this.actor,
     });
-    this.closePipWindow({ reason: "closeButton" });
   },
 
   fullscreenModeToggle() {
@@ -765,8 +829,8 @@ let Player = {
       this.deferredResize = {
         left: window.screenX,
         top: window.screenY,
-        width: window.innerWidth,
-        height: window.innerHeight,
+        width: window.outerWidth,
+        height: window.outerHeight,
       };
       document.body.requestFullscreen();
     }
@@ -894,8 +958,8 @@ let Player = {
    */
   determineCurrentQuadrant() {
     // Determine center coordinates of window.
-    let windowCenterX = window.screenX + window.innerWidth / 2;
-    let windowCenterY = window.screenY + window.innerHeight / 2;
+    let windowCenterX = window.screenX + window.outerWidth / 2;
+    let windowCenterY = window.screenY + window.outerHeight / 2;
     let quadrant = null;
     let halfWidth = window.screen.availLeft + window.screen.availWidth / 2;
     let halfHeight = window.screen.availTop + window.screen.availHeight / 2;
@@ -989,7 +1053,11 @@ let Player = {
     let quadrant = this.determineCurrentQuadrant();
     let dragAction = this.determineDirectionDragged();
 
-    if (event.metaKey && AppConstants.platform == "macosx" && dragAction) {
+    if (
+      ((event.ctrlKey && AppConstants.platform !== "macosx") ||
+        (event.metaKey && AppConstants.platform === "macosx")) &&
+      dragAction
+    ) {
       // Moving logic based on current quadrant and direction of drag.
       switch (quadrant) {
         case TOP_RIGHT_QUADRANT:
@@ -1082,15 +1150,11 @@ let Player = {
     if (!this.isFullscreen) {
       this.isCurrentHover = false;
       if (
-        !this.controls.getAttribute("showing") &&
-        !this.controls.getAttribute("keying") &&
-        !this.controls.getAttribute("donthide")
+        !this.controls.hasAttribute(SHOWING_ATTRIBUTE) &&
+        !this.controls.hasAttribute(KEYING_ATTRIBUTE) &&
+        !this.controls.hasAttribute(DONTHIDE_ATTRIBUTE)
       ) {
-        this.actor.sendAsyncMessage("PictureInPicture:HideVideoControls", {
-          isFullscreen: this.isFullscreen,
-          isVideoControlsShowing: false,
-          playerBottomControlsDOMRect: null,
-        });
+        this.hideVideoControls();
       }
     }
   },
@@ -1134,7 +1198,7 @@ let Player = {
    * @param {Event} event
    *  Event context data object
    */
-  onResize(event) {
+  onResize() {
     this.toggleSubtitlesSettingsPanel({ forceHide: true });
     this.resizeDebouncer.disarm();
     this.resizeDebouncer.arm();
@@ -1146,8 +1210,8 @@ let Player = {
    * @param {Event} event
    *  Event context data object
    */
-  onCommand(event) {
-    this.closePipWindow({ reason: "shortcut" });
+  onCommand() {
+    this.closePipWindow({ reason: "Shortcut" });
   },
 
   get controls() {
@@ -1216,6 +1280,23 @@ let Player = {
       ? `pictureinpicture-pause-btn`
       : `pictureinpicture-play-btn`;
     this.setupTooltip("playpause", strId);
+
+    if (
+      !this._isInitialized ||
+      // Currently, controls will always be visible on hover for non-fullscreen.
+      // Only ensure we hide controls after playing state update for fullscreen,
+      // by not bothering to call revealControls.
+      (!this.isFullscreen && this.isCurrentHover) ||
+      this.controls.hasAttribute(KEYING_ATTRIBUTE)
+    ) {
+      return;
+    }
+
+    if (!isPlaying) {
+      this.revealControls(true);
+    } else {
+      this.revealControls(false);
+    }
   },
 
   _isMuted: false,
@@ -1264,17 +1345,13 @@ let Player = {
    *   The data to pass to telemetry when the event is recorded.
    */
   recordEvent(type, args) {
-    Services.telemetry.recordEvent(
-      "pictureinpicture",
-      type,
-      "player",
-      this.id,
-      args
-    );
+    args.value = this.id;
+    Glean.pictureinpicture[type + "Player"].record(args);
   },
 
   /**
    * Send a message to PiPChild to adjust the subtitles position
+   * so that subtitles are visible when showing video controls.
    */
   showVideoControls() {
     // offsetParent returns null when the element or any ancestor has display: none
@@ -1287,6 +1364,18 @@ let Player = {
     });
   },
 
+  /**
+   * Send a message to PiPChild to adjust the subtitles position
+   * so that subtitles take up remaining space when hiding video controls.
+   */
+  hideVideoControls() {
+    this.actor.sendAsyncMessage("PictureInPicture:HideVideoControls", {
+      isFullscreen: this.isFullscreen,
+      isVideoControlsShowing: false,
+      playerBottomControlsDOMRect: null,
+    });
+  },
+
   /**
    * Makes the player controls visible.
    *
@@ -1300,7 +1389,7 @@ let Player = {
     clearTimeout(this.showingTimeout);
     this.showingTimeout = null;
 
-    this.controls.setAttribute("showing", true);
+    this.controls.setAttribute(SHOWING_ATTRIBUTE, true);
 
     if (!this.isFullscreen) {
       // revealControls() is called everytime we hover over fullscreen pip window.
@@ -1316,19 +1405,15 @@ let Player = {
         if (this.isFullscreen && isHoverOverControlItem) {
           return;
         }
-        this.controls.removeAttribute("showing");
+        this.controls.removeAttribute(SHOWING_ATTRIBUTE);
 
         if (
           !this.isFullscreen &&
           !this.isCurrentHover &&
-          !this.controls.getAttribute("keying") &&
-          !this.controls.getAttribute("donthide")
+          !this.controls.hasAttribute(KEYING_ATTRIBUTE) &&
+          !this.controls.hasAttribute(DONTHIDE_ATTRIBUTE)
         ) {
-          this.actor.sendAsyncMessage("PictureInPicture:HideVideoControls", {
-            isFullscreen: false,
-            isVideoControlsShowing: false,
-            playerBottomControlsDOMRect: null,
-          });
+          this.hideVideoControls();
         }
       }, CONTROLS_FADE_TIMEOUT_MS);
     }
