@@ -238,7 +238,7 @@ var popupTests = [
       // rollup this way.
       // synthesizeMouse(gTrigger, 0, -12, { });
     },
-    result(testname, step) {
+    result(testname) {
       is(gMenuPopup.anchorNode, null, testname + " anchorNode");
       is(gMenuPopup.triggerNode, null, testname + " triggerNode");
       checkClosed("trigger", testname);
@@ -259,7 +259,6 @@ var popupTests = [
       "start_after",
       "end_before",
       "end_after",
-      "after_pointer",
       "overlap",
       "topleft topleft",
       "topcenter topleft",
@@ -272,6 +271,8 @@ var popupTests = [
       "topleft bottomright",
       "bottomcenter bottomright",
       "rightcenter topright",
+      "bottomcenter topcenter",
+      "rightcenter leftcenter",
     ],
     test(testname, step) {
       gExpectedTriggerNode = "notset";
@@ -299,7 +300,6 @@ var popupTests = [
       "start_after",
       "end_before",
       "end_after",
-      "after_pointer",
       "overlap",
       "topleft topleft",
       "topcenter topleft",
@@ -357,7 +357,6 @@ var popupTests = [
       "start_after",
       "end_before",
       "end_after",
-      "after_pointer",
       "overlap",
     ],
     test(testname, step) {
@@ -399,7 +398,6 @@ var popupTests = [
       "start_after",
       "end_before",
       "end_after",
-      "after_pointer",
       "overlap",
     ],
     test(testname, step) {
@@ -445,7 +443,6 @@ var popupTests = [
       "start_after",
       "end_before",
       "end_after",
-      "after_pointer",
       "overlap",
     ],
     test(testname, step) {
@@ -510,7 +507,6 @@ var popupTests = [
       "start_after",
       "end_before",
       "end_after",
-      "after_pointer",
       "overlap",
       "topcenter topleft",
       "topright bottomright",
@@ -530,7 +526,7 @@ var popupTests = [
     // event to openPopup to check the trigger node.
     testname: "open popup anchored with override",
     events: ["popupshowing thepopup 0010", "popupshown thepopup"],
-    test(testname, step) {
+    test() {
       // attribute overrides the position passed in
       gMenuPopup.setAttribute("position", "end_after");
       gExpectedTriggerNode = gCachedEvent.target;
@@ -544,7 +540,7 @@ var popupTests = [
         gCachedEvent
       );
     },
-    result(testname, step) {
+    result(testname) {
       gExpectedTriggerNode = null;
       is(gMenuPopup.anchorNode, gTrigger, testname + " anchorNode");
       is(
@@ -562,7 +558,7 @@ var popupTests = [
       "popuphidden thepopup",
       "DOMMenuInactive thepopup",
     ],
-    test(testname, step) {
+    test(testname) {
       synthesizeKey("KEY_Escape");
       checkClosed("trigger", testname);
     },
@@ -572,12 +568,12 @@ var popupTests = [
     testname: "open popup anchored with offsets",
     events: ["popupshowing thepopup", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test() {
       // attribute is empty so does not override
       gMenuPopup.setAttribute("position", "");
       gMenuPopup.openPopup(gTrigger, "before_start", 5, 10, true, true);
     },
-    result(testname, step) {
+    result(testname) {
       compareEdge(gTrigger, gMenuPopup, "before_start", 5, 10, testname);
     },
   },
@@ -586,10 +582,10 @@ var popupTests = [
     // to the viewport.
     testname: "open popup unanchored",
     events: ["popupshowing thepopup", "popupshown thepopup"],
-    test(testname, step) {
+    test() {
       gMenuPopup.openPopup(null, "after_start", 6, 8, false);
     },
-    result(testname, step) {
+    result(testname) {
       var rect = gMenuPopup.getBoundingClientRect();
       ok(
         rect.left == 6 && rect.top == 8 && rect.right && rect.bottom,
@@ -605,11 +601,11 @@ var popupTests = [
       "popuphiding thepopup",
       "popuphidden thepopup",
     ],
-    test(testname, step) {
+    test() {
       var item3 = document.getElementById("item3");
       synthesizeMouse(item3, 4, 4, {});
     },
-    result(testname, step) {
+    result(testname) {
       checkClosed("trigger", testname);
     },
   },
@@ -623,18 +619,18 @@ var popupTests = [
       "popuphidden thepopup",
       "DOMMenuInactive thepopup",
     ],
-    test(testname, step) {
+    test() {
       gMenuPopup.hidePopup();
     },
   },
   {
     testname: "open popup at screen",
     events: ["popupshowing thepopup", "popupshown thepopup"],
-    test(testname, step) {
+    test() {
       gExpectedTriggerNode = "notset";
       gMenuPopup.openPopupAtScreen(gScreenX + 24, gScreenY + 20, false);
     },
-    result(testname, step) {
+    result(testname) {
       gExpectedTriggerNode = null;
       is(gMenuPopup.anchorNode, null, testname + " anchorNode");
       is(gMenuPopup.triggerNode, null, testname + " triggerNode");
@@ -668,7 +664,7 @@ var popupTests = [
   {
     testname: "open context popup at screen",
     events: ["popupshowing thepopup 0010", "popupshown thepopup"],
-    test(testname, step) {
+    test() {
       gExpectedTriggerNode = gCachedEvent.target;
       gMenuPopup.openPopupAtScreen(
         gScreenX + 8,
@@ -677,7 +673,7 @@ var popupTests = [
         gCachedEvent
       );
     },
-    result(testname, step) {
+    result(testname) {
       gExpectedTriggerNode = null;
       is(gMenuPopup.anchorNode, null, testname + " anchorNode");
       is(
@@ -871,7 +867,7 @@ var popupTests = [
     testname: "open context popup at screen with all modifiers set",
     events: ["popupshowing thepopup 1111", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test() {
       gMenuPopup.openPopupAtScreen(
         gScreenX + 8,
         gScreenY + 16,
@@ -883,10 +879,10 @@ var popupTests = [
   {
     testname: "open popup with open property",
     events: ["popupshowing thepopup", "popupshown thepopup"],
-    test(testname, step) {
+    test() {
       openMenu(gTrigger);
     },
-    result(testname, step) {
+    result(testname) {
       checkOpen("trigger", testname);
       if (gIsMenu) {
         compareEdge(gTrigger, gMenuPopup, "after_start", 0, 0, testname);
@@ -900,10 +896,10 @@ var popupTests = [
       "DOMMenuItemActive submenu",
       "popupshown submenupopup",
     ],
-    test(testname, step) {
+    test() {
       openMenu(document.getElementById("submenu"));
     },
-    result(testname, step) {
+    result(testname) {
       checkOpen("trigger", testname);
       checkOpen("submenu", testname);
       // XXXndeakin
@@ -927,17 +923,17 @@ var popupTests = [
     test() {
       gMenuPopup.hidePopup();
     },
-    result(testname, step) {
+    result(testname) {
       checkClosed("trigger", testname);
       checkClosed("submenu", testname);
     },
   },
   {
     testname: "open submenu with open property without parent open",
-    test(testname, step) {
+    test() {
       openMenu(document.getElementById("submenu"));
     },
-    result(testname, step) {
+    result(testname) {
       checkClosed("trigger", testname);
       checkClosed("submenu", testname);
     },
@@ -948,11 +944,11 @@ var popupTests = [
       return gIsMenu;
     },
     events: ["popupshowing thepopup", "popupshown thepopup"],
-    test(testname, step) {
+    test() {
       gMenuPopup.setAttribute("position", "before_start");
       openMenu(gTrigger);
     },
-    result(testname, step) {
+    result(testname) {
       compareEdge(gTrigger, gMenuPopup, "before_start", 0, 0, testname);
     },
   },
@@ -966,10 +962,10 @@ var popupTests = [
       "popuphidden thepopup",
       "DOMMenuInactive thepopup",
     ],
-    test(testname, step) {
+    test() {
       closeMenu(gTrigger, gMenuPopup);
     },
-    result(testname, step) {
+    result(testname) {
       checkClosed("trigger", testname);
     },
   },
@@ -980,13 +976,13 @@ var popupTests = [
     },
     events: ["popupshowing thepopup", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test() {
       gMenuPopup.setAttribute("position", "start_after");
       gMenuPopup.setAttribute("popupanchor", "topright");
       gMenuPopup.setAttribute("popupalign", "bottomright");
       openMenu(gTrigger);
     },
-    result(testname, step) {
+    result(testname) {
       compareEdge(gTrigger, gMenuPopup, "start_after", 0, 0, testname);
     },
   },
@@ -997,13 +993,13 @@ var popupTests = [
     },
     events: ["popupshowing thepopup", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test() {
       gMenuPopup.removeAttribute("position");
       gMenuPopup.setAttribute("popupanchor", "bottomright");
       gMenuPopup.setAttribute("popupalign", "topright");
       openMenu(gTrigger);
     },
-    result(testname, step) {
+    result(testname) {
       compareEdge(gTrigger, gMenuPopup, "after_end", 0, 0, testname);
       gMenuPopup.removeAttribute("popupanchor");
       gMenuPopup.removeAttribute("popupalign");
@@ -1016,11 +1012,11 @@ var popupTests = [
     },
     events: ["popupshowing thepopup", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test() {
       gTrigger.focus();
       synthesizeKey("KEY_ArrowDown", { altKey: !platformIsMac() });
     },
-    result(testname, step) {
+    result(testname) {
       checkOpen("trigger", testname);
       checkActive(gMenuPopup, "", testname);
     },
@@ -1031,11 +1027,11 @@ var popupTests = [
       return gIsMenu;
     },
     events: ["popupshowing thepopup", "popupshown thepopup"],
-    test(testname, step) {
+    test() {
       gTrigger.focus();
       synthesizeKey("KEY_ArrowUp", { altKey: !platformIsMac() });
     },
-    result(testname, step) {
+    result(testname) {
       checkOpen("trigger", testname);
       checkActive(gMenuPopup, "", testname);
     },
@@ -1053,11 +1049,11 @@ var popupTests = [
       "popuphiding thepopup",
       "popuphidden thepopup",
     ],
-    test(testname, step) {
+    test() {
       synthesizeKey("KEY_ArrowDown");
       synthesizeKey("KEY_Enter");
     },
-    result(testname, step) {
+    result(testname) {
       checkClosed("trigger", testname);
     },
   },
@@ -1068,11 +1064,11 @@ var popupTests = [
     },
     events: ["popupshowing thepopup", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test() {
       gTrigger.focus();
       synthesizeKey(platformIsMac() ? " " : "KEY_F4");
     },
-    result(testname, step) {
+    result(testname) {
       checkOpen("trigger", testname);
       checkActive(gMenuPopup, "", testname);
     },
@@ -1083,7 +1079,7 @@ var popupTests = [
     condition() {
       return gIsMenu;
     },
-    test(testname, step) {
+    test() {
       gTrigger.focus();
       if (platformIsMac()) {
         synthesizeKey("KEY_F4", { altKey: true });
@@ -1091,7 +1087,7 @@ var popupTests = [
         synthesizeKey("", { metaKey: true });
       }
     },
-    result(testname, step) {
+    result(testname) {
       checkClosed("trigger", testname);
     },
   },
@@ -1100,11 +1096,11 @@ var popupTests = [
     condition() {
       return gIsMenu;
     },
-    test(testname, step) {
+    test() {
       gTrigger.setAttribute("disabled", "true");
       synthesizeMouse(gTrigger, 4, 4, {});
     },
-    result(testname, step) {
+    result(testname) {
       checkClosed("trigger", testname);
       gTrigger.removeAttribute("disabled");
     },
@@ -1114,11 +1110,11 @@ var popupTests = [
     testname: "openPopup with object argument",
     events: ["popupshowing thepopup 0000", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test(testname) {
       gMenuPopup.openPopup(gTrigger, { position: "before_start", x: 5, y: 7 });
       checkOpen("trigger", testname);
     },
-    result(testname, step) {
+    result(testname) {
       var triggerrect = gTrigger.getBoundingClientRect();
       var popuprect = gMenuPopup.getBoundingClientRect();
       is(
@@ -1137,7 +1133,7 @@ var popupTests = [
     testname: "openPopup with object argument with event",
     events: ["popupshowing thepopup 1000", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test(testname) {
       gMenuPopup.openPopup(gTrigger, {
         position: "after_start",
         x: 0,
@@ -1151,10 +1147,10 @@ var popupTests = [
     testname: "openPopup with no arguments",
     events: ["popupshowing thepopup", "popupshown thepopup"],
     autohide: "thepopup",
-    test(testname, step) {
+    test() {
       gMenuPopup.openPopup();
     },
-    result(testname, step) {
+    result(testname) {
       let isMenu = gTrigger.type == "menu";
       // With no arguments, open in default menu position
       var triggerrect = gTrigger.getBoundingClientRect();
@@ -1182,7 +1178,7 @@ var popupTests = [
       "DOMMenuItemActive submenu",
       "popupshown submenupopup",
     ],
-    test(testname, step) {
+    test(testname) {
       gMenuPopup.openPopup(gTrigger, "after_start", 0, 0, false, true);
       document
         .getElementById("submenupopup")
@@ -1194,7 +1190,7 @@ var popupTests = [
   {
     // remove the content nodes for the popup
     testname: "remove content",
-    test(testname, step) {
+    test() {
       var submenupopup = document.getElementById("submenupopup");
       submenupopup.remove();
       var popup = document.getElementById("thepopup");
